@@ -23,7 +23,8 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-`insideBrackets` <- function(x, type = "{", invert = FALSE) {
+`insideBrackets` <- function(x, type = "[", invert = FALSE) {
+    x <- recreate(substitute(x))
     typematrix <- matrix(c("{", "[", "(", "}", "]", ")", "{}", "[]", "()"), nrow = 3)
     tml <- which(typematrix == type, arr.ind = TRUE)[1]
     if (is.na(tml)) {
@@ -35,7 +36,8 @@
     result <- gsub("\\*|\\+", "", unlist(strsplit(gsub("\\s+", " ", result), split = " ")))
     return(result[result != ""])
 }
-`outsideBrackets` <- function(x, type = "{") {
+`outsideBrackets` <- function(x, type = "[") {
+    x <- recreate(substitute(x))
     typematrix <- matrix(c("{", "[", "(", "}", "]", ")", "{}", "[]", "()"), nrow = 3)
     tml <- which(typematrix == type, arr.ind = TRUE)[1]
     if (is.na(tml)) {
@@ -47,6 +49,7 @@
     return(result[result != ""])
 }
 `curlyBrackets` <- function(x, outside = FALSE) {
+    x <- recreate(substitute(x))
     x <- paste(x, collapse = "+")
     regexp <- "\\{[[:alnum:]|,|;]+\\}"
     x <- gsub("[[:space:]]", "", x)
@@ -59,7 +62,22 @@
         return(gsub("\\{|\\}|\\*", "", res))
     }
 }
+`squareBrackets` <- function(x, outside = FALSE) {
+    x <- recreate(substitute(x))
+    x <- paste(x, collapse = "+")
+    regexp <- "\\[[[:alnum:]|,|;]+\\]"
+    x <- gsub("[[:space:]]", "", x)
+    res <- regmatches(x, gregexpr(regexp, x), invert = outside)[[1]]
+    if (outside) {
+        res <- gsub("\\*", "", unlist(strsplit(res, split="\\+")))
+        return(res[res != ""])
+    }
+    else {
+        return(gsub("\\[|\\]|\\*", "", res))
+    }
+}
 `roundBrackets` <- function(x, outside = FALSE) {
+    x <- recreate(substitute(x))
     regexp <- "\\(([^)]+)\\)"
     x <- gsub("[[:space:]]", "", x)
     res <- regmatches(x, gregexpr(regexp, x), invert = outside)[[1]]
@@ -72,8 +90,10 @@
     }
 }
 `expandBrackets` <- function(expression, snames = "", noflevels = NULL, collapse = "*") {
+    expression <- recreate(substitute(expression))
     snames <- splitstr(snames)
-    multivalue <- any(grepl("[{|}]", expression))
+    multivalue <- any(grepl("\\[|\\]|\\{|\\}", expression))
+    curly <- grepl("[{]", expression)
     sl <- ifelse(identical(snames, ""), FALSE, ifelse(all(nchar(snames) == 1), TRUE, FALSE))
     getbl <- function(expression, snames = "", noflevels = NULL) {
         bl <- splitMainComponents(gsub("[[:space:]]", "", expression))
@@ -95,7 +115,8 @@
     }
     bl <- getbl(expression, snames = snames, noflevels = noflevels)
     if (length(bl) == 0) return("")
-    expressions <- translate(paste(unlist(lapply(bl, paste, collapse = collapse)), collapse = " + "), snames = snames, noflevels = noflevels)
+    bl <- paste(unlist(lapply(bl, paste, collapse = collapse)), collapse = " + ")
+    expressions <- translate(bl, snames = snames, noflevels = noflevels)
     snames <- colnames(expressions)
     redundant <- logical(nrow(expressions))
     if (nrow(expressions) > 1) {
@@ -126,7 +147,7 @@
         for (i in seq(length(snames))) {
             if (x[i] != -1) {
                 if (multivalue) {
-                    result <- c(result, paste(snames[i], "{", x[i], "}", sep = ""))
+                    result <- c(result, paste(snames[i], ifelse(curly, "{", "["), x[i], ifelse(curly, "}", "]"), sep = ""))
                 }
                 else {
                     if (x[i] == 0) {
