@@ -25,17 +25,22 @@
 
 `factorize` <- 
 function(input, snames = "", noflevels = NULL, pos = FALSE, ...) {
-    other.args <- list(...)
-    scollapse <- ifelse(is.element("scollapse", names(other.args)), other.args$scollapse, FALSE) 
-    `pasteit` <- function(mat, comrows, cols, comvals, snames = "", mv = FALSE, collapse = "*") {
+    input <- recreate(substitute(input))
+    if (identical(input, character(0))) {
+        return(invisible(input))
+    }
+    snames <- recreate(substitute(snames))
+    dots <- list(...)
+    scollapse <- ifelse(is.element("scollapse", names(dots)), dots$scollapse, FALSE) 
+    `pasteit` <- function(mat, comrows, cols, comvals, snames = "", mv = FALSE, collapse = "*", curly = FALSE) {
         if (!missing(cols)) {
             temp <- mat[comrows, -cols, drop = FALSE]
             if (mv) {
-                cf <- paste(colnames(mat)[cols], "{", comvals, "}", sep = "")
+                cf <- paste(colnames(mat)[cols], ifelse(curly, "{", "["), comvals, ifelse(curly, "}", "]"), sep = "")
                 rowsf <- lapply(seq(nrow(temp)), function(x) {
                     fname <- colnames(temp)
                     x <- temp[x, ]
-                    return(paste(fname, "{", x, "}", sep = "")[x >= 0])
+                    return(paste(fname, ifelse(curly, "{", "["), x, ifelse(curly, "}", "]"), sep = "")[x >= 0])
                 })
             }
             else {
@@ -82,7 +87,7 @@ function(input, snames = "", noflevels = NULL, pos = FALSE, ...) {
             if (mv) {
                 pasted <- paste(sapply(seq(nrow(mat)), function(x) {
                     x <- mat[x, ]
-                    paste(paste(names(x), "{", x, "}", sep = "")[x >= 0], collapse = "*")
+                    paste(paste(names(x), ifelse(curly, "{", "["), x, ifelse(curly, "}", "]"), sep = "")[x >= 0], collapse = "*")
                 }), collapse = " + ")
             }
             else {
@@ -95,7 +100,7 @@ function(input, snames = "", noflevels = NULL, pos = FALSE, ...) {
         }
         return(pasted)
     }
-    `getFacts` <- function(mat, snames = "", mv = FALSE, collapse = "*") {
+    `getFacts` <- function(mat, snames = "", mv = FALSE, collapse = "*", curly = FALSE) {
         cfound <- FALSE
         result <- list()
         for (cc in seq(ncol(mat))) {
@@ -117,7 +122,8 @@ function(input, snames = "", noflevels = NULL, pos = FALSE, ...) {
                                                 comvals = unname(uniq[i, ]),
                                                 snames = snames,
                                                 mv = mv,
-                                                collapse = collapse)
+                                                collapse = collapse,
+                                                curly = curly)
                             if (sum(rows) < nrow(mat)) {
                                 result[[length(result) + 1]] <- Recall(mat[!rows, , drop = FALSE], snames = snames, mv = mv, collapse = collapse)
                                 names(result)[length(result)] <- pasted
@@ -133,11 +139,11 @@ function(input, snames = "", noflevels = NULL, pos = FALSE, ...) {
         }
         if (!cfound) {
             result <- list(NA)
-            names(result) <- pasteit(mat = mat, snames = snames, mv = mv, collapse = collapse)
+            names(result) <- pasteit(mat = mat, snames = snames, mv = mv, collapse = collapse, curly = curly)
         }
         return(result)
     }
-    `getSol` <- function(sol, pos = FALSE, noflevels = NULL, snames = "", mv = FALSE, collapse = "*") {
+    `getSol` <- function(sol, pos = FALSE, noflevels = NULL, snames = "", mv = FALSE, collapse = "*", curly = FALSE) {
         pospos <- FALSE
         sol <- lapply(unique(lapply(sol, sort)), function(x) {
             x <- strsplit(gsub("@1 \\+ 1", "", x), split = "@")
@@ -146,7 +152,7 @@ function(input, snames = "", noflevels = NULL, pos = FALSE, ...) {
                 for (i in seq(length(x))) {
                     xi <- unlist(strsplit(x[i], split = " \\+ "))
                     for (j in seq(length(xi))) {
-                        xi[j] <- pasteit(translate(xi[j], snames = snames), snames = snames, mv = mv, collapse = collapse)
+                        xi[j] <- pasteit(translate(xi[j], snames = snames), snames = snames, mv = mv, collapse = collapse, curly = curly)
                     }
                     x[i] <- paste(xi, collapse = " + ")
                 }
@@ -190,35 +196,45 @@ function(input, snames = "", noflevels = NULL, pos = FALSE, ...) {
                 }
             }
             else {
-                x <- sort(sapply(x, function(y) {
+                x <- sapply(x, function(y) {
                     if (length(y) == 1) {
                         return(y)
                     }
                     res <- simplify(y[2], snames = snames, noflevels = noflevels, scollapse = identical(collapse, "*"))
+                    if (identical(res, character(0))) {
+                        return(res)
+                    }
                     if (res == "") {
                         return(y[1])
                     }
                     paste(y[1], collapse, "(", res, ")", sep = "")
-                }))
+                })
+                if (any(unlist(lapply(x, length)) == 0)) {
+                    return(character(0))
+                }
+                x <- sort(x)
             }
             return(x)
         })
+        if (any(unlist(lapply(sol, length)) == 0)) {
+            return(character(0))
+        }
         sol <- unlist(lapply(unique(sol), function(x) {
             paste(x, collapse = " + ")
         }))
         return(sol)
     }
-    `factorizeit` <- function(x, pos = FALSE, noflevels = NULL, snames = "", mv = FALSE) {
+    `factorizeit` <- function(x, pos = FALSE, noflevels = NULL, snames = "", mv = FALSE, curly = FALSE) {
         if (grepl("[(|)]", x)) {
             x <- expandBrackets(x, snames = snames, noflevels = noflevels)
         }
         trexp <- translate(x, snames = snames, noflevels = noflevels)
         snames <- colnames(trexp)
         collapse <- ifelse(any(nchar(snames) > 1) | mv | scollapse | grepl("[*]", x), "*", "")
-        facts <- names(unlist(getFacts(mat = trexp, snames = snames, mv = mv, collapse = collapse)))
+        facts <- names(unlist(getFacts(mat = trexp, snames = snames, mv = mv, collapse = collapse, curly = curly)))
         facts <- lapply(facts, function(x) unlist(strsplit(x, split = "[.]")))
         facts <- unique(lapply(facts, sort))
-        getSol(facts, pos = pos, noflevels = noflevels, snames = snames, mv = mv, collapse = collapse)
+        getSol(facts, pos = pos, noflevels = noflevels, snames = snames, mv = mv, collapse = collapse, curly = curly)
     }
     isol <- NULL
     if (methods::is(input, "QCA_min")) {
@@ -255,9 +271,10 @@ function(input, snames = "", noflevels = NULL, pos = FALSE, ...) {
         if (!identical(snames, "")) {
             snames <- splitstr(snames)
         }
-        mv <- any(grepl("[{]", unlist(input)))
+        mv <- any(grepl("\\[|\\{", unlist(input)))
+        curly <- any(grepl("\\{", unlist(input)))
         result <- lapply(input, function(x) {
-            factorizeit(x, pos = pos, snames = snames, noflevels = noflevels, mv = mv)
+            factorizeit(x, pos = pos, snames = snames, noflevels = noflevels, mv = mv, curly = curly)
         })
         names(result) <- unname(input)
         if (!identical(snames, "")) {
